#!/usr/bin/env Rscript

######################################
## Load configuration file
cmd_args <- commandArgs(trailingOnly = T)
if (length(cmd_args) != 1){
  cat(paste0("[",Sys.time() ,"] ERROR - Usage: CLONET.LoadAndCLeanDataForTEP.R <config_file>\n"))
  quit()
}

######################################
## Plot intro message
ClonET_version<-"1.0.0"
cat("\n")
cat(paste0("[",Sys.time() ,"] CLONET v",ClonET_version,"\n",sep=""))
cat(paste0("[",Sys.time() ,"]   Tool to clean data for evolution analysis\n",sep=""))
wd <- getwd()
cat(paste0("[",Sys.time() ,"] Working directory ",wd,"\n\n",sep=""))


######################################
## Load configuration file
configFile <- cmd_args[1]
cat(paste0("[",Sys.time() ,"] Load configuration file: ",configFile,"\n",sep=""))
source(configFile)

######################################
## Check R version
minMajorVersion <- 2
minMinorVersion <- 7 
currect_R_version <- R.version
if ( ( as.numeric(currect_R_version$major) < minMajorVersion)  || 
       ( as.numeric(currect_R_version$major) == minMajorVersion && as.numeric(currect_R_version$minor) < minMinorVersion) ){
  cat(paste0("[",Sys.time() ,"] ERROR - CLONET require at least version ",minMajorVersion,".",minMinorVersion," but you have version ",currect_R_version$major,".",currect_R_version$minor, "\n"))
  quit()
}


######################################
## Check installed packages
cat(paste0("[",Sys.time() ,"] Check installed packages\n",sep=""))
pkg.list = c("parallel","pso","dgof","sets")
pkg.list.match=match(pkg.list,installed.packages()[,"Package"],nomatch=FALSE)
pkg.list.match = !as.logical(pkg.list.match)
if(sum(pkg.list.match)>0) { install.packages(pkg.list[pkg.list.match],repos="http://cran.r-project.org") } 
library(parallel,warn.conflicts=F)
library(pso,warn.conflicts=F)
library(dgof,warn.conflicts=F)
library(sets,warn.conflicts=F)
options(scipen=99999)

######################################
## Load CLONET basic functions
cat(paste0("[",Sys.time() ,"] Load CLONET basic functions\n",sep=""))
source(paste0(path_to_CLONET_functions,"Functions/CLONET.BasicFunctions.R"))

######################################
## Check output directory
output_DIR <- dirname(OutRdata_file)
cat(paste("[",Sys.time() ,"] Output directory ",output_DIR,"\n",sep=""))
if (!file.exists(output_DIR)){
  cat(paste("[",Sys.time() ,"] WARNING - Output directory does not exit and will be created\n",sep=""))
  dir.create(path=output_DIR)
}




#####################################
## Computing per per gene SCNA clonality value
cat("\n\n######################################################################\n")
cat(paste("[",Sys.time() ,"] Clean data for evolution analysis\n",sep=""))
cat(paste("[",Sys.time() ,"] Per segment clonality ",cnData_file,"\n",sep=""))
cat(paste("[",Sys.time() ,"] Per PM clonality ",pmData_file,"\n",sep=""))
cat(paste("[",Sys.time() ,"] Per RR clonality ",rrData_file,"\n",sep=""))
cat(paste("[",Sys.time() ,"] Per gene SCNA clonality ",geneTableSCNA_file,"\n",sep=""))

## Load data
cat(paste("[",Sys.time() ,"] Load data\n",sep=""))
cnData <- data.frame(NULL)
pmData <- data.frame(NULL)
rrData <- data.frame(NULL)
geneData.CN <- data.frame(NULL)
# Load tables
if (cnData_file != "" && file.exists(cnData_file)){
  cnData <- readTab(cnData_file)
}
if (pmData_file != "" && file.exists(pmData_file)){
  pmData <- readTab(pmData_file)
}
if (rrData_file != "" && file.exists(rrData_file)){
  rrData <- readTab(rrData_file)
}
if (geneTableSCNA_file != "" && file.exists(geneTableSCNA_file)){
  geneData.CN <- readTab(geneTableSCNA_file)
}

errorTable <- readTab(errorTable_file)
adm.Global <- readTab(Adm.global_file)



### Clean the data
## Load data
cat(paste("[",Sys.time() ,"] Clean data\n",sep=""))
#CN: keep only low adm and remove bad samples and not analysed
cnDataGood <- cnData[which(
  !cnData$sample %in% samplesToRemove & 
    cnData$adm.global <= Adm.global.max & 
    cnData$clonality.status !=  "not.analysed" & 
    cnData$nsnp >=  minSNPs & 
    cnData$cov >=  minCov ) ,]


#PM: keep only low adm integerCN = 2 and remove not analysed
if (length(pmData) > 0){
  pmDataGood <- pmData[which(
    !pmData$sample %in% samplesToRemove & pmData$adm.global <= Adm.global.max & 
      pmData$clonality.status != "not.analysed" &
      pmData$integer.cn == 2 &
      (pmData$ref + pmData$alt) >= minCov.PM &
      pmData$alt >= minAltCov.PM ) ,]
}


if (length(rrData)> 0){
  rrDataGood <- rrData[which(
    !rrData$sample %in% samplesToRemove & rrData$adm.global <= Adm.global.max & 
      rrData$clonality.status != "not.analysed" &
      rrData$minCov >= minCov.BP ) ,]
}


#Gene in SCNA: keep only low adm, low error, mutated genes
if (length(geneData.CN)> 0){
  geneDataCN.Good <- geneData.CN[which(
    !geneData.CN$sample %in% samplesToRemove & 
      geneData.CN$adm.global <= Adm.global.max  &
      geneData.CN$clonality.status != "not.analysed" &
      geneData.CN$nSNPs >=  minSNPs & 
      geneData.CN$cov >=  minCov ) ,]
}



#remove duplicated in geneData (a gene may be part of two segments)
if (length(geneData.CN)> 0){
  geneDataCN.Good$unique <- paste0(geneDataCN.Good$Gene.id," ",geneDataCN.Good$sample)
  dupIds <- which(duplicated(geneDataCN.Good$unique))
  dupNames <- unique(geneDataCN.Good$unique[dupIds])
  name <- dupNames[1]
  
  length(which(geneDataCN.Good$clonality.status == "not.analysed"))
  
  geneDataCN.Good.list <- mclapply(X=c(1:nrow(geneDataCN.Good)),
                                   function(i,geneData.CN,dupNames){
                                     if (!geneData.CN$unique[i] %in% dupNames){
                                       return(geneData.CN[i,])
                                     }else{
                                       this.ids <- which(geneData.CN$unique == geneData.CN$unique[i])
                                       if ( this.ids[which.max(geneData.CN$perc.overlap[this.ids])]  == i){
                                         return(geneData.CN[i,])
                                       }else{
                                         geneData.CN$clonality.status[i] <- "not.analysed"
                                         return(geneData.CN[i,])
                                       }
                                     }
                                   },geneDataCN.Good,dupNames,mc.cores=ncores
  )
  
  geneDataCN.Good.unique <-  fromListToDF(geneDataCN.Good.list)
  geneDataCN.Good <- geneDataCN.Good.unique[which(geneDataCN.Good.unique$clonality.status != "not.analysed"),]
  rm(geneDataCN.Good.list)
  rm(geneDataCN.Good.unique)
  #length(which(geneDataCN.Good.unique$clonality.status == "not.analysed"))
  
  geneDataCN.Good.Loss <- geneDataCN.Good[which(geneDataCN.Good$integer.cn %in% lossCNs),]
  geneDataCN.Good.Gain <- geneDataCN.Good[which(geneDataCN.Good$integer.cn %in% gainCNs),]
}


save.image(file=OutRdata_file)

