#!/usr/bin/env Rscript

######################################
## Load configuration file
cmd_args <- commandArgs(trailingOnly = T)
if (length(cmd_args) != 1){
  cat(paste0("[",Sys.time() ,"] ERROR - Usage: CLONET.createEvolutionGraph.R <config_file>\n"))
  quit()
}

######################################
## Plot intro message
ClonET_version<-"1.0.0"
cat("\n")
cat(paste0("[",Sys.time() ,"] CLONET v",ClonET_version,"\n",sep=""))
cat(paste0("[",Sys.time() ,"]   Tool to create tumor evolution path\n",sep=""))
wd <- getwd()
cat(paste0("[",Sys.time() ,"] Working directory ",wd,"\n\n",sep=""))


######################################
## Load configuration file
configFile <- cmd_args[1]
cat(paste0("[",Sys.time() ,"] Load configuration file: ",configFile,"\n",sep=""))
source(configFile)

######################################
## Check R version
minMajorVersion <- 2
minMinorVersion <- 7 
currect_R_version <- R.version
if ( ( as.numeric(currect_R_version$major) < minMajorVersion)  || 
       ( as.numeric(currect_R_version$major) == minMajorVersion && as.numeric(currect_R_version$minor) < minMinorVersion) ){
  cat(paste0("[",Sys.time() ,"] ERROR - CLONET require at least version ",minMajorVersion,".",minMinorVersion," but you have version ",currect_R_version$major,".",currect_R_version$minor, "\n"))
  quit()
}


######################################
## Check installed packages
cat(paste0("[",Sys.time() ,"] Check installed packages\n",sep=""))
pkg.list = c("parallel","pso","dgof","sets","reshape2","igraph")
pkg.list.match=match(pkg.list,installed.packages()[,"Package"],nomatch=FALSE)
pkg.list.match = !as.logical(pkg.list.match)
if(sum(pkg.list.match)>0) { install.packages(pkg.list[pkg.list.match],repos="http://cran.r-project.org") } 
library(parallel,warn.conflicts=F)
library(pso,warn.conflicts=F)
library(dgof,warn.conflicts=F)
library(sets,warn.conflicts=F)
library(reshape2,warn.conflicts=F)
library(igraph,warn.conflicts=F)
options(scipen=99999)

######################################
## Load CLONET basic functions
cat(paste0("[",Sys.time() ,"] Load CLONET basic functions\n",sep=""))
source(paste0(path_to_CLONET_functions,"Functions/CLONET.BasicFunctions.R"))
source(paste0(path_to_CLONET_functions,"Functions/CLONET.EvolutionPathFunctions.R"))


######################################
## Check output directory
output_DIR <- dirname(OutGraph_file)
cat(paste("[",Sys.time() ,"] Output directory ",output_DIR,"\n",sep=""))
if (!file.exists(output_DIR)){
  cat(paste("[",Sys.time() ,"] WARNING - Output directory does not exit and will be created\n",sep=""))
  dir.create(path=output_DIR)
}



#####################################
## Computing per PM clonality value
cat("\n\n######################################################################\n")
cat(paste("[",Sys.time() ,"] Compute tumor evolution path\n",sep=""))
cat(paste("[",Sys.time() ,"] CLONET cleaned data ",CleanData_RData,"\n",sep=""))

cat(paste("[",Sys.time() ,"] Output graph file name ",OutGraph_file,"\n",sep=""))


cat(paste("[",Sys.time() ,"] Load data\n",sep=""))

## cleaned data
load(CleanData_RData)

## check if good genes are specified
if (ListOfGenes_file != ""){
  goodGenes <- unique(read.delim(ListOfGenes_file,as.is=T)[,1])  
}else{
  goodGenes <- unique(c(geneDataCN.Good.Gain$Gene.id,geneDataCN.Good.Loss$Gene.id,rrDataGood$Gene.id,pmDataGood$Gene.id))
}

if (genesInfo_file != ""){
  genesInfo <- readTab(genesInfo_file,colName=T)
  genesInfo <- genesInfo[-which(duplicated(genesInfo[,4])),]
  rownames(genesInfo) <- genesInfo[,4]
  
  # save genes info for good genes
  goodGenes <- goodGenes[which(goodGenes %in% genesInfo$HUGO)]  
}else{
  genesInfo <- data.frame(row.names=NULL)
}


#### subset clonality tables by good genes
clonalityGoodGenes.loss <- geneDataCN.Good.Loss[which(geneDataCN.Good.Loss$Gene.id %in% goodGenes),]
nSamples.loss <- length(unique(geneDataCN.Good.Loss$sample))
clonalityGoodGenes.gain <- geneDataCN.Good.Gain[which(geneDataCN.Good.Gain$Gene.id %in% goodGenes),]
nSamples.gain <- length(unique(geneDataCN.Good.Gain$sample))
clonalityGoodGenes.pm <- pmDataGood[which(pmDataGood$Gene.id %in% goodGenes),]
nSamples.pm <- length(unique(pmDataGood$sample))
clonalityGoodGenes.rr <- pmDataGood[which(rrDataGood$Gene.id %in% goodGenes),]
nSamples.rr <- length(unique(rrDataGood$sample))

## comptue stats about clonal/subclonal status of genes
cat(paste("[",Sys.time() ,"] Compute Statistics about clonal/subclonal status of genes\n",sep=""))
genesStats.loss <- getMutationsStats(clonalityGoodGenes.loss,nSamples=nSamples.loss,clonalStatus=clonalStatus,subclonalStatus = subclonalStatus)
genesStats.gain <- getMutationsStats(clonalityGoodGenes.gain,nSamples=nSamples.gain,clonalStatus=clonalStatus,subclonalStatus = subclonalStatus )
genesStats.pm <- getMutationsStats(clonalityGoodGenes.pm,nSamples=nSamples.pm,clonalStatus=clonalStatus,subclonalStatus = subclonalStatus )
genesStats.rr <- getMutationsStats(clonalityGoodGenes.rr,nSamples=nSamples.rr,clonalStatus=clonalStatus,subclonalStatus = subclonalStatus )

## compute precendences between pair of genes
cat(paste("[",Sys.time() ,"] Compute precendences between pair of genes\n",sep=""))
countClonalityDependecy_function <- "countClonalityDependecy.fast3"
list_of_clonalityTables <- list(loss=clonalityGoodGenes.loss,gain=clonalityGoodGenes.gain ,pm=clonalityGoodGenes.pm)
genesEvolutionPath <- countClonalityDependecy.aggreateMutationType(
  countClonalityDependecy_function=countClonalityDependecy_function,
  list_of_clonalityTables=list_of_clonalityTables, clonalStatus,subclonalStatus,ncores=ncores)


## create the graph
# create a palette for nodes
cat(paste("[",Sys.time() ,"] Create the evolution graph\n",sep=""))
palSub <- colorRampPalette(colors=c(rgb(217,255,252,maxColorValue=255),rgb(127, 255, 212,maxColorValue=255)))(255)
palClo <- colorRampPalette(colors=c(rgb(89,168,15,maxColorValue=255),rgb(217,255,189,maxColorValue=255)))(255)
vertexPal <- c(palClo[1],palClo[255],palSub[1],palSub[255])


clonality.stats.list <- list(loss=genesStats.loss,gain=genesStats.gain,pm=genesStats.pm)
dGraph <- plotDependencyGraph.aggreateMutationType(clonality.dep= genesEvolutionPath,plot=F,
                                                   clonality.stats.list=clonality.stats.list,
                                                   vertexPal=vertexPal,
                                                   plotElabels=T,
                                                   name.separator="~",
                                                   minDep=1,
                                                   nodesInfo=genesInfo)

if (length(E(dGraph)) > 0){
  cat(paste("[",Sys.time() ,"] Write results\n",sep=""))
  write.graph(dGraph, OutGraph_file, format="graphml")
  
  Today <- format(Sys.time(), "%a, %b %d")
  Hour <- format(Sys.time(), "%H:%M")
  cat(paste0("Computation end on ",Today," at ",Hour,"\n"))
  
}else{
  cat(paste("[",Sys.time() ,"] CLONET did not find any valid dependency\n",sep=""))
  Today <- format(Sys.time(), "%a, %b %d")
  Hour <- format(Sys.time(), "%H:%M")
  cat(paste0("Computation end on ",Today," at ",Hour,"\n"))
}












